'use strict';

var fs = require('fs');
var converter = require('json-2-csv');
var prompt = require('prompt');
prompt.message = prompt.delimeter = '';

var API = require('./api');
var api = new API();
var utils = require('./utils');

var nextOrders = function nextOrders(orders, cursor, resolve) {
  // No more pages. Resolve and return.
  if (cursor === null) {
    return resolve(orders);
  }

  api.orders(cursor).then(function (response) {
    nextOrders(orders.concat(response.results), response.next, resolve);
  });
};

module.exports = {
  login: function login() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var username = _ref.username;
    var password = _ref.password;

    return new Promise(function (resolve) {
      prompt.override = { username: username, password: password };

      var properties = {
        username: { default: process.env.ROBINHOOD_USERNAME, required: true },
        password: { default: process.env.ROBINHOOD_PASSWORD, required: true, hidden: true }
      };

      var handleResponse = function handleResponse(_err, res) {
        resolve(api.login(res));
      };

      prompt.start();
      prompt.get({ properties: properties }, handleResponse);
    });
  },

  getOrders: function getOrders() {
    return new Promise(function (resolve) {
      nextOrders([], '', resolve);
    });
  },

  getSymbols: function getSymbols(orders) {
    return Promise.all(orders.map(function (order) {
      return api.instrument(utils.pluckLastOfPath(order.instrument)).then(function (_ref2) {
        var symbol = _ref2.symbol;
        return Object.assign({}, order, { symbol: symbol });
      });
    }));
  },

  getExecutions: function getExecutions(orders) {
    return orders.reduce(function (e, order) {
      return e.concat(order.executions.map(function (execution) {
        return {
          symbol: order.symbol,
          shares: execution.quantity,
          price: utils.formatCurrency(execution.price),
          transaction_type: order.side,
          commission: utils.formatCurrency(order.fees),
          date_executed: execution.timestamp
        };
      }));
    }, []);
  },

  convertToCsv: function convertToCsv(transactions) {
    return new Promise(function (resolve, reject) {
      return converter.json2csv(transactions, function (err, csv) {
        if (err) {
          reject(err);
        } else {
          resolve(csv);
        }
      });
    });
  },

  printCsv: function printCsv(content, destination) {
    if (destination === undefined) {
      console.log(content); // eslint-disable-line no-console
    } else {
      fs.writeFileSync(destination, content, 'utf8');
    }
  }
};